/*
 * Copyright (C) 2008 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.dixheure;

import java.io.IOException;
import java.util.List;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.res.AssetFileDescriptor;
import android.content.res.Resources;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.MediaPlayer.OnErrorListener;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.Vibrator;
import android.telephony.PhoneStateListener;
import android.telephony.TelephonyManager;

import com.dixheure.io.ListDeezerDataReader;
import com.dixheure.model.Radio;
import com.dixheure.model.Track;
import com.google.gson.Gson;
import com.loopj.android.http.AsyncHttpClient;
import com.loopj.android.http.AsyncHttpResponseHandler;

/**
 * Manages alarms and vibe. Runs as a service so that it can continue to play
 * if another activity overrides the AlarmAlert dialog.
 */
public class AlarmKlaxon extends Service  {

	/** Play alarm up to 10 minutes before silencing */
	private static final int ALARM_TIMEOUT_SECONDS = 10 * 60;

	public static final String TRACK_EXTRA="track";
	
	public static final String TRACK_ACTION="com.dixheure.TRACK_ACTION";
	
	public static final String TRACK_WAKEUP="com.dixheure.TRACK_WAKEUP";
	
	private final static String URL="http://api.deezer.com/2.0/";

	private static final long[] sVibratePattern = new long[] { 500, 500 };

	private boolean mPlaying = false;
	private Vibrator mVibrator;
	private MediaPlayer mMediaPlayer;
	private Alarm mCurrentAlarm;
	private long mStartTime;
	private TelephonyManager mTelephonyManager;
	private int mInitialCallState;

	// Internal messages
	private static final int KILLER = 1000;
	private Handler mHandler = new Handler() {
		public void handleMessage(Message msg) {
			switch (msg.what) {
			case KILLER:
				if (Log.LOGV) {
					Log.v("*********** Alarm killer triggered ***********");
				}
				sendKillBroadcast((Alarm) msg.obj);
				stopSelf();
				break;
			}
		}
	};

	private PhoneStateListener mPhoneStateListener = new PhoneStateListener() {
		@Override
		public void onCallStateChanged(int state, String ignored) {
			// The user might already be in a call when the alarm fires. When
			// we register onCallStateChanged, we get the initial in-call state
			// which kills the alarm. Check against the initial call state so
			// we don't kill the alarm during a call.
			if (state != TelephonyManager.CALL_STATE_IDLE
					&& state != mInitialCallState) {
				sendKillBroadcast(mCurrentAlarm);
				stopSelf();
			}
		}
	};

	@Override
	public void onCreate() {
		mVibrator = (Vibrator) getSystemService(Context.VIBRATOR_SERVICE);
		// Listen for incoming calls to kill the alarm.
		mTelephonyManager =
				(TelephonyManager) getSystemService(Context.TELEPHONY_SERVICE);
		mTelephonyManager.listen(
				mPhoneStateListener, PhoneStateListener.LISTEN_CALL_STATE);
		AlarmAlertWakeLock.acquireCpuWakeLock(this);
	}

	@Override
	public void onDestroy() {
		stop();
		// Stop listening for incoming calls.
		mTelephonyManager.listen(mPhoneStateListener, 0);
		AlarmAlertWakeLock.releaseCpuLock();
	}

	@Override
	public IBinder onBind(Intent intent) {
		return null;
	}

	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		// No intent, tell the system not to restart us.
		if (intent == null) {
			stopSelf();
			return START_NOT_STICKY;
		}

		final Alarm alarm = intent.getParcelableExtra(
				Alarms.ALARM_INTENT_EXTRA);

		if (alarm == null) {
			Log.v("AlarmKlaxon failed to parse the alarm from the intent");
			stopSelf();
			return START_NOT_STICKY;
		}

		if (mCurrentAlarm != null) {
			sendKillBroadcast(mCurrentAlarm);
		}

		play(alarm);
		mCurrentAlarm = alarm;
		// Record the initial call state here so that the new alarm has the
		// newest state.
		mInitialCallState = mTelephonyManager.getCallState();

		return START_STICKY;
	}

	private void sendKillBroadcast(Alarm alarm) {
		long millis = System.currentTimeMillis() - mStartTime;
		int minutes = (int) Math.round(millis / 60000.0);
		Intent alarmKilled = new Intent(Alarms.ALARM_KILLED);
		alarmKilled.putExtra(Alarms.ALARM_INTENT_EXTRA, alarm);
		alarmKilled.putExtra(Alarms.ALARM_KILLED_TIMEOUT, minutes);
		sendBroadcast(alarmKilled);
	}

	// Volume suggested by media team for in-call alarms.
	private static final float IN_CALL_VOLUME = 0.125f;


	private int i=0;


	private void playTrack (List<Track> tracks){

		try {

			final Track track=tracks.get(i);
			
			mMediaPlayer.setDataSource(track.getPreview());
			mMediaPlayer.setAudioStreamType(AudioManager.STREAM_MUSIC);

			mMediaPlayer.prepareAsync();
			mMediaPlayer.setOnPreparedListener(new MediaPlayer.OnPreparedListener() {

				@Override
				public void onPrepared(MediaPlayer mp) {
					
					Intent intent=new Intent(TRACK_ACTION);
					intent.putExtra(TRACK_EXTRA, gson.toJson(track));
					sendBroadcast(intent);
					
					mMediaPlayer.start();
				}
			});

			i=(i+1)%tracks.size();
		} catch (IOException e) {
			playRollback();
		}
	}
	
	private final 	Gson gson=new Gson();;

	private void play(final Alarm alarm) {
		// stop() checks to see if we are already playing.
		stop();

		System.out.println("AlarmKlaxon.play() " + alarm.id + " alert " + alarm.alert+" "+alarm.radio_id+" "+alarm.radio);
		Log.v("AlarmKlaxon.play() " + alarm.id + " alert " + alarm.alert+" "+alarm.radio_id);

		if (!alarm.silent) {
			Uri alert = alarm.alert;
			// Fall back on the default alarm if the database does not have an
			// alarm stored.
			if (alert == null) {
				alert = RingtoneManager.getDefaultUri(
						RingtoneManager.TYPE_ALARM);
				if (Log.LOGV) {
					Log.v("Using default alarm: " + alert.toString());
				}
			}

			mMediaPlayer = new MediaPlayer();
			mMediaPlayer.setOnErrorListener(new OnErrorListener() {
				public boolean onError(MediaPlayer mp, int what, int extra) {
					Log.e("Error occurred while playing audio.");
					mp.stop();
					mp.release();
					mMediaPlayer = null;
					return true;
				}
			});


			try {

				if (mTelephonyManager.getCallState()
						!= TelephonyManager.CALL_STATE_IDLE) {
					Log.v("Using the in-call alarm");
					mMediaPlayer.setVolume(IN_CALL_VOLUME, IN_CALL_VOLUME);
					setDataSourceFromResource(getResources(), mMediaPlayer,
							R.raw.in_call_alarm);
				}
				else {
					
			
					
					Radio radio=gson.fromJson(alarm.radio, Radio.class);
					
					AsyncHttpClient client = new AsyncHttpClient();
					client.get(URL+"radio/"+radio.getId()+"/tracks", new AsyncHttpResponseHandler() {

						@Override
						public void onSuccess(String response) {
							
							System.out.println("-->"+response);

							final List<Track> tracks = new ListDeezerDataReader<Track>(Track.class).readList(response);

							if(tracks==null || tracks.size()==0){
								playRollback();

							}
							else {

								mMediaPlayer.setOnCompletionListener(new MediaPlayer.OnCompletionListener() {

									@Override
									public void onCompletion(MediaPlayer mp) {
										mMediaPlayer.reset();
										playTrack(tracks);
									}
								});
								mPlaying = true;
								playTrack(tracks);
							}
						}

						@Override
						public void onFailure(Throwable throwable, String response) {
							playRollback();
						}

					});

				}

			} catch (Exception ex) {
				Log.v("Using the fallback ringtone");
				playRollback();
			}
		}

		/* Start the vibrator after everything is ok with the media player */
		if (alarm.vibrate) {
			mVibrator.vibrate(sVibratePattern, 0);
		} else {
			mVibrator.cancel();
		}

		enableKiller(alarm);
		mStartTime = System.currentTimeMillis();
	}


	public void playRollback (){
		try {

			if(mPlaying){
				mMediaPlayer.release();
				mMediaPlayer.reset();
			}


			setDataSourceFromResource(getResources(), mMediaPlayer,R.raw.in_call_alarm);
			startAlarm(mMediaPlayer);
		} catch (Exception ex2) {
			// At this point we just don't play anything.
			Log.e("Failed to play fallback ringtone", ex2);
		}
	}

	// Do the common stuff when starting the alarm.
	private void startAlarm(MediaPlayer player)
			throws java.io.IOException, IllegalArgumentException,
			IllegalStateException {

		mPlaying = true;
		final AudioManager audioManager = (AudioManager)getSystemService(Context.AUDIO_SERVICE);
		// do not play alarms if stream volume is 0
		// (typically because ringer mode is silent).
		if (audioManager.getStreamVolume(AudioManager.STREAM_ALARM) != 0) {
			player.setAudioStreamType(AudioManager.STREAM_ALARM);
			player.setLooping(true);
			player.prepare();
			player.start();
			
			Intent intent=new Intent(TRACK_ACTION);
			sendBroadcast(intent);
		}
	}

	private void setDataSourceFromResource(Resources resources,
			MediaPlayer player, int res) throws java.io.IOException {
		AssetFileDescriptor afd = resources.openRawResourceFd(res);
		if (afd != null) {
			player.setDataSource(afd.getFileDescriptor(), afd.getStartOffset(),
					afd.getLength());
			afd.close();
		}
	}

	/**
	 * Stops alarm audio and disables alarm if it not snoozed and not
	 * repeating
	 */
	public void stop() {
		if (Log.LOGV) Log.v("AlarmKlaxon.stop()");
		if (mPlaying) {
			mPlaying = false;

			// Stop audio playing
			if (mMediaPlayer != null) {
				mMediaPlayer.stop();
				mMediaPlayer.release();
				mMediaPlayer = null;
			}

			// Stop vibrator
			mVibrator.cancel();
		}
		disableKiller();
	}

	/**
	 * Kills alarm audio after ALARM_TIMEOUT_SECONDS, so the alarm
	 * won't run all day.
	 *
	 * This just cancels the audio, but leaves the notification
	 * popped, so the user will know that the alarm tripped.
	 */
	private void enableKiller(Alarm alarm) {
		mHandler.sendMessageDelayed(mHandler.obtainMessage(KILLER, alarm),
				1000 * ALARM_TIMEOUT_SECONDS);
	}

	private void disableKiller() {
		mHandler.removeMessages(KILLER);
	}


}
